#!/usr/local/bin/python
#-------------------------------------------------------------------------------
# Name:        contact.py
# Purpose:     Contact form for hirebrett.com.
#
# Author:      Brett Gmoser
#
# Created:     2014-08-25
# Copyright:   (c) Brett Gmoser 2013
# Licence:     Proprietary and internal.
#-------------------------------------------------------------------------------

import smtplib, cgi, sys, re, os
import logging

import smtplib
from email.MIMEMultipart import MIMEMultipart
from email.MIMEBase import MIMEBase
from email.MIMEText import MIMEText
from email.Utils import COMMASPACE, formatdate
from email import Encoders

# Where the notification e-mails will be sent.
NOTIFICATION_EMAILS = [ 'bgmoser@codexterous.com' ]

# Mail server to use.
MAIL_SERVER = 'mail.codexterous.com'

# List of languages we support for the CAPTCHA, and optional messages for them.
LANGUAGES = [
  [ r'A#', None, False ],
  [ r'A-0 System', None, False ],
  [ r'A+', None, False ],
  [ r'A\+\+', None, False ],
  [ r'ABAP', None, False ],
  [ r'ABC', None, False ],
  [ r'ABC ALGOL', None, False ],
  [ r'ABLE', None, False ],
  [ r'ABSET', None, False ],
  [ r'ABSYS', None, False ],
  [ r'ACC', None, False ],
  [ r'Accent', None, False ],
  [ r'Ace DASL', None, False ],
  [ r'ACL2', None, False ],
  [ r'ACT-III', None, False ],
  [ r'Action!', None, False ],
  [ r'ActionScript', None, False ],
  [ r'Ada', None, False ],
  [ r'Adenine', None, False ],
  [ r'Agda', None, False ],
  [ r'Agilent VEE', None, False ],
  [ r'Agora', None, False ],
  [ r'AIMMS', None, False ],
  [ r'Alef', None, False ],
  [ r'ALF', None, False ],
  [ r'ALGOL 58', None, False ],
  [ r'ALGOL 60', None, False ],
  [ r'ALGOL 68', None, False ],
  [ r'ALGOL W', None, False ],
  [ r'ALGOL', None, False ],
  [ r'Alice', None, False ],
  [ r'Alma-0', None, False ],
  [ r'AmbientTalk', None, False ],
  [ r'Amiga E', None, False ],
  [ r'AMOS', None, False ],
  [ r'AMPL', None, False ],
  [ r'APL', None, False ],
  [ r'App Inventor', None, False ],
  [ r'AppleScript', None, False ],
  [ r'Arc', None, False ],
  [ r'ARexx', None, False ],
  [ r'Argus', None, False ],
  [ r'AspectJ', None, False ],
  [ r'Assembly language', None, False ],
  [ r'Assembly', None, False ],
  [ r'ASM', None, False ],
  [ r'ATS', None, False ],
  [ r'Ateji PX', None, False ],
  [ r'AutoHotkey', None, False ],
  [ r'Autocoder', None, False ],
  [ r'AutoIt', None, False ],
  [ r'(AutoLISP)|(Visual LISP)', None, False ],
  [ r'Averest', None, False ],
  [ r'AWK', None, False ],
  [ r'Axum', None, False ],
  [ r'B', None, False ],
  [ r'B', None, False ],
  [ r'Babbage', None, False ],
  [ r'BAIL', None, False ],
  [ r'Bash', None, False ],
  [ r'BASIC', None, False ],
  [ r'bc', None, False ],
  [ r'BCPL', None, False ],
  [ r'BeanShell', None, False ],
  [ r'Batch', None, False ],
  [ r'Bertrand', None, False ],
  [ r'BETA', None, False ],
  [ r'Bigwig', None, False ],
  [ r'Bistro', None, False ],
  [ r'BitC', None, False ],
  [ r'BLISS', None, False ],
  [ r'Blue', None, False ],
  [ r'Bon', None, False ],
  [ r'Boo', None, False ],
  [ r'Boomerang', None, False ],
  [ r'Bourne shell', None, False ],
  [ r'bash', None, False ],
  [ r'BREW', None, False ],
  [ r'BPEL', None, False ],
  [ r'BuildProfessional', None, False ],
  [ r'C', None, False ],
  [ r'C--', None, False ],
  [ r'C\+\+.*', 'C++! Nice language!', False ],
  [ r'C#', None, False ],
  [ r'C/AL', None, False ],
  [ r'Cach\? ObjectScript', None, False ],
  [ r'Cach\?', None, False ],
  [ r'Cach', None, False ],
  [ r'C Shell', None, False ],
  [ r'csh', None, False ],
  [ r'Caml', None, False ],
  [ r'Candle', None, False ],
  [ r'Cayenne', None, False ],
  [ r'CDuce', None, False ],
  [ r'Cecil', None, False ],
  [ r'Cel', None, False ],
  [ r'Cesil', None, False ],
  [ r'Ceylon', None, False ],
  [ r'CFML', None, False ],
  [ r'Cg', None, False ],
  [ r'Ch', None, False ],
  [ r'Chapel', None, False ],
  [ r'CHAIN', None, False ],
  [ r'Charity', None, False ],
  [ r'Charm', None, False ],
  [ r'Chef', None, False ],
  [ r'CHILL', None, False ],
  [ r'CHIP-8', None, False ],
  [ r'chomski', None, False ],
  [ r'ChucK', None, False ],
  [ r'CICS', None, False ],
  [ r'Cilk', None, False ],
  [ r'CL', None, False ],
  [ r'Claire', None, False ],
  [ r'Clarion', None, False ],
  [ r'Clean', None, False ],
  [ r'Clipper', None, False ],
  [ r'CLIST', None, False ],
  [ r'Clojure', None, False ],
  [ r'CLU', None, False ],
  [ r'CMS-2', None, False ],
  [ r'COBOL', None, False ],
  [ r'Cobra', None, False ],
  [ r'CODE', None, False ],
  [ r'CoffeeScript', None, False ],
  [ r'Cola', None, False ],
  [ r'ColdC', None, False ],
  [ r'ColdFusion', None, False ],
  [ r'COMAL', None, False ],
  [ r'Combined Programming Language', None, False ],
  [ r'CPL', None, False ],
  [ r'COMIT', None, False ],
  [ r'CIL', None, False ],
  [ r'Common Lisp', None, False ],
  [ r'CL', None, False ],
  [ r'COMPASS', None, False ],
  [ r'Component Pascal', None, False ],
  [ r'(Constraint Handling Rules)|(CHR)', None, False ],
  [ r'CHR', None, False ],
  [ r'Converge', None, False ],
  [ r'Cool', None, False ],
  [ r'Coq', None, False ],
  [ r'Coral 66', None, False ],
  [ r'Corn', None, False ],
  [ r'CorVision', None, False ],
  [ r'COWSEL', None, False ],
  [ r'CPL', None, False ],
  [ r'csh', None, False ],
  [ r'CSP', None, False ],
  [ r'Csound', None, False ],
  [ r'CUDA', None, False ],
  [ r'Curl', None, False ],
  [ r'Curry', None, False ],
  [ r'Cyclone', None, False ],
  [ r'Cython', None, False ],
  [ r'D', None, False ],
  [ r'D', None, False ],
  [ r'DASL', None, False ],
  [ r'Dart', None, False ],
  [ r'DataFlex', None, False ],
  [ r'Datalog', None, False ],
  [ r'DATATRIEVE', None, False ],
  [ r'dBase', None, False ],
  [ r'dc', None, False ],
  [ r'DCL', None, False ],
  [ r'Deesel', None, False ],
  [ r'Delphi', None, False ],
  [ r'DCL', None, False ],
  [ r'DinkC', None, False ],
  [ r'DIBOL', None, False ],
  [ r'Dog', None, False ],
  [ r'Draco', None, False ],
  [ r'DRAKON', None, False ],
  [ r'Dylan', None, False ],
  [ r'DYNAMO', None, False ],
  [ r'E', None, False ],
  [ r'E', None, False ],
  [ r'E#', None, False ],
  [ r'Ease', None, False ],
  [ r'Easy PL/I', None, False ],
  [ r'Easy Programming Language', None, False ],
  [ r'EASYTRIEVE PLUS', None, False ],
  [ r'ECMAScript', None, False ],
  [ r'Edinburgh IMP', None, False ],
  [ r'EGL', None, False ],
  [ r'Eiffel', None, False ],
  [ r'ELAN', None, False ],
  [ r'Elixir', None, False ],
  [ r'Elm', None, False ],
  [ r'Emacs Lisp', None, False ],
  [ r'Emerald', None, False ],
  [ r'Epigram', None, False ],
  [ r'EPL', None, False ],
  [ r'Erlang', None, False ],
  [ r'es', None, False ],
  [ r'Escapade', None, False ],
  [ r'Escher', None, False ],
  [ r'ESPOL', None, False ],
  [ r'Esterel', None, False ],
  [ r'Etoys', None, False ],
  [ r'Euclid', None, False ],
  [ r'Euler', None, False ],
  [ r'Euphoria', None, False ],
  [ r'EusLisp', None, False ],
  [ r'CMS EXEC', None, False ],
  [ r'EXEC 2', None, False ],
  [ r'Executable UML', None, False ],
  [ r'F', None, False ],
  [ r'F', None, False ],
  [ r'F#', None, False ],
  [ r'Factor', None, False ],
  [ r'Falcon', None, False ],
  [ r'Fancy', None, False ],
  [ r'Fantom', None, False ],
  [ r'FAUST', None, False ],
  [ r'Felix', None, False ],
  [ r'Ferite', None, False ],
  [ r'FFP', None, False ],
  [ r'Fj?lnir', None, False ],
  [ r'FL', None, False ],
  [ r'Flavors', None, False ],
  [ r'Flex', None, False ],
  [ r'FLOW-MATIC', None, False ],
  [ r'FOCAL', None, False ],
  [ r'FOCUS', None, False ],
  [ r'FOIL', None, False ],
  [ r'FORMAC', None, False ],
  [ r'@Formula', None, False ],
  [ r'Forth', None, False ],
  [ r'Fortran', None, False ],
  [ r'Fortress', None, False ],
  [ r'FoxBase', None, False ],
  [ r'FoxPro', None, False ],
  [ r'FP', None, False ],
  [ r'FPr', None, False ],
  [ r'Franz Lisp', None, False ],
  [ r'F-Script', None, False ],
  [ r'FSProg', None, False ],
  [ r'G', None, False ],
  [ r'G', None, False ],
  [ r'Google Apps Script', None, False ],
  [ r'Game Maker Language', None, False ],
  [ r'GameMonkey Script', None, False ],
  [ r'GAMS', None, False ],
  [ r'GAP', None, False ],
  [ r'G-code', None, False ],
  [ r'Genie', None, False ],
  [ r'GDL', None, False ],
  [ r'Gibiane', None, False ],
  [ r'GJ', None, False ],
  [ r'GEORGE', None, False ],
  [ r'GLSL', None, False ],
  [ r'GNU E', None, False ],
  [ r'GM', None, False ],
  [ r'Go', None, False ],
  [ r'Go!', None, False ],
  [ r'GOAL', None, False ],
  [ r'Godel', None, False ],
  [ r'Godiva', None, False ],
  [ r'GOM', None, False ],
  [ r'Goo', None, False ],
  [ r'Gosu', None, False ],
  [ r'GOTRAN', None, False ],
  [ r'GPSS', None, False ],
  [ r'GraphTalk', None, False ],
  [ r'GRASS', None, False ],
  [ r'Groovy', None, False ],
  [ r'H', None, False ],
  [ r'Hack', None, False ],
  [ r'HAL/S', None, False ],
  [ r'Hamilton C shell', None, False ],
  [ r'Harbour', None, False ],
  [ r'Hartmann pipelines', None, False ],
  [ r'Haskell', None, False ],
  [ r'Haxe', None, False ],
  [ r'High Level Assembly', None, False ],
  [ r'HLSL', None, False ],
  [ r'Hop', None, False ],
  [ r'Hope', None, False ],
  [ r'Hugo', None, False ],
  [ r'Hume', None, False ],
  [ r'HyperTalk', None, False ],
  [ r'I', None, False ],
  [ r'IBM Basic assembly language', None, False ],
  [ r'IBM HAScript', None, False ],
  [ r'IBM Informix-4GL', None, False ],
  [ r'IBM RPG', None, False ],
  [ r'ICI', None, False ],
  [ r'Icon', None, False ],
  [ r'Id', None, False ],
  [ r'IDL', None, False ],
  [ r'Idris', None, False ],
  [ r'IMP', None, False ],
  [ r'Inform', None, False ],
  [ r'Io', None, False ],
  [ r'Ioke', None, False ],
  [ r'IPL', None, False ],
  [ r'IPTSCRAE', None, False ],
  [ r'ISLISP', None, False ],
  [ r'ISPF', None, False ],
  [ r'ISWIM', None, False ],
  [ r'J', None, False ],
  [ r'J', None, False ],
  [ r'J#', None, False ],
  [ r'J\+\+', None, False ],
  [ r'JADE', None, False ],
  [ r'Jako', None, False ],
  [ r'JAL', None, False ],
  [ r'Janus', None, False ],
  [ r'JASS', None, False ],
  [ r'Java', None, False ],
  [ r'JavaScript', None, False ],
  [ r'JCL', None, False ],
  [ r'JEAN', None, False ],
  [ r'Join Java', None, False ],
  [ r'JOSS', None, False ],
  [ r'Joule', None, False ],
  [ r'JOVIAL', None, False ],
  [ r'Joy', None, False ],
  [ r'JScript( .NET)?', None, False ],
  [ r'JavaFX Script', None, False ],
  [ r'Julia', None, False ],
  [ r'K', None, False ],
  [ r'Kaleidoscope', None, False ],
  [ r'Karel', None, False ],
  [ r'Karel\+\+', None, False ],
  [ r'KEE', None, False ],
  [ r'KIF', None, False ],
  [ r'Kojo', None, False ],
  [ r'Kotlin', None, False ],
  [ r'KRC', None, False ],
  [ r'KRL', None, False ],
  [ r'KRL', None, False ],
  [ r'KRYPTON', None, False ],
  [ r'ksh', None, False ],
  [ r'L', None, False ],
  [ r'L#( .NET)?', None, False ],
  [ r'LabVIEW', None, False ],
  [ r'Ladder', None, False ],
  [ r'Lagoona', None, False ],
  [ r'LANSA', None, False ],
  [ r'Lasso', None, False ],
  [ r'LaTeX', None, False ],
  [ r'Lava', None, False ],
  [ r'LC-3', None, False ],
  [ r'Leda', None, False ],
  [ r'Legoscript', None, False ],
  [ r'LIL', None, False ],
  [ r'LilyPond', None, False ],
  [ r'Limbo', None, False ],
  [ r'Limnor', None, False ],
  [ r'LINC', None, False ],
  [ r'Lingo', None, False ],
  [ r'Linoleum', None, False ],
  [ r'LIS', None, False ],
  [ r'LISA', None, False ],
  [ r'Lisaac', None, False ],
  [ r'Lisp', None, False ],
  [ r'Lite-C', None, False ],
  [ r'Lithe', None, False ],
  [ r'Little b', None, False ],
  [ r'Logo', None, False ],
  [ r'Logtalk', None, False ],
  [ r'LPC', None, False ],
  [ r'LSE', None, False ],
  [ r'LSL', None, False ],
  [ r'LiveCode', None, False ],
  [ r'LiveScript', None, False ],
  [ r'Lua', None, False ],
  [ r'Lucid', None, False ],
  [ r'Lustre', None, False ],
  [ r'LYaPAS', None, False ],
  [ r'Lynx', None, False ],
  [ r'M', None, False ],
  [ r'M2001', None, False ],
  [ r'M4', None, False ],
  [ r'Machine code', None, False ],
  [ r'MAD', None, False ],
  [ r'MAD/I', None, False ],
  [ r'Magik', None, False ],
  [ r'Magma', None, False ],
  [ r'make', None, False ],
  [ r'Maple', None, False ],
  [ r'MAPPER', None, False ],
  [ r'MARK-IV', None, False ],
  [ r'Mary', None, False ],
  [ r'(MASM)|(Microsoft Assembly x86)', None, False ],
  [ r'Mathematica', None, False ],
  [ r'MATLAB', None, False ],
  [ r'Maxima', None, False ],
  [ r'Max', None, False ],
  [ r'MaxScript', None, False ],
  [ r'(Maya)|(MEL)', None, False ],
  [ r'MDL', None, False ],
  [ r'Mercury', None, False ],
  [ r'Mesa', None, False ],
  [ r'Metacard', None, False ],
  [ r'Metafont', None, False ],
  [ r'MetaL', None, False ],
  [ r'Microcode', None, False ],
  [ r'MicroScript', None, False ],
  [ r'MIIS', None, False ],
  [ r'MillScript', None, False ],
  [ r'MIMIC', None, False ],
  [ r'Mirah', None, False ],
  [ r'Miranda', None, False ],
  [ r'MIVA Script', None, False ],
  [ r'ML', None, False ],
  [ r'Moby', None, False ],
  [ r'Model 204', None, False ],
  [ r'Modelica', None, False ],
  [ r'Modula', None, False ],
  [ r'Modula-2', None, False ],
  [ r'Modula-3', None, False ],
  [ r'Mohol', None, False ],
  [ r'MOO', None, False ],
  [ r'Mortran', None, False ],
  [ r'Mouse', None, False ],
  [ r'MPD', None, False ],
  [ r'MSIL', None, False ],
  [ r'MSL', None, False ],
  [ r'MUMPS', None, False ],
  [ r'N', None, False ],
  [ r'NASM', None, False ],
  [ r'NATURAL', None, False ],
  [ r'Napier88', None, False ],
  [ r'Neko', None, False ],
  [ r'Nemerle', None, False ],
  [ r'nesC', None, False ],
  [ r'NESL', None, False ],
  [ r'Net.Data', None, False ],
  [ r'NetLogo', None, False ],
  [ r'NetRexx', None, False ],
  [ r'NewLISP', None, False ],
  [ r'NEWP', None, False ],
  [ r'Newspeak', None, False ],
  [ r'NewtonScript', None, False ],
  [ r'NGL', None, False ],
  [ r'Nial', None, False ],
  [ r'Nice', None, False ],
  [ r'Nickle', None, False ],
  [ r'NPL', None, False ],
  [ r'(Not eXactly C)|(NXC)', None, False ],
  [ r'(Not Quite C)|(NQC)', None, False ],
  [ r'NSIS', None, False ],
  [ r'Nu', None, False ],
  [ r'NWScript', None, False ],
  [ r'NXT-G', None, False ],
  [ r'O', None, False ],
  [ r'o:XML', None, False ],
  [ r'Oak', None, False ],
  [ r'Oberon', None, False ],
  [ r'Obix', None, False ],
  [ r'OBJ2', None, False ],
  [ r'Object Lisp', None, False ],
  [ r'ObjectLOGO', None, False ],
  [ r'Object REXX', None, False ],
  [ r'Object Pascal', None, False ],
  [ r'Obj(ective)?[- ]?C', None, False ],
  [ r'Obj(ective)?[- ]?J', None, False ],
  [ r'Obliq', None, False ],
  [ r'Obol', None, False ],
  [ r'OCaml', None, False ],
  [ r'occam', None, False ],
  [ r'occam-?', None, False ],
  [ r'Octave', None, False ],
  [ r'OmniMark', None, False ],
  [ r'Onyx', None, False ],
  [ r'Opa', None, False ],
  [ r'Opal', None, False ],
  [ r'OpenCL', None, False ],
  [ r'OpenEdge ABL', None, False ],
  [ r'OPL', None, False ],
  [ r'OPS5', None, False ],
  [ r'OptimJ', None, False ],
  [ r'Orc', None, False ],
  [ r'ORCA', None, False ],
  [ r'Oriel', None, False ],
  [ r'Orwell', None, False ],
  [ r'Oxygene', None, False ],
  [ r'Oz', None, False ],
  [ r'P', None, False ],
  [ r'P#', None, False ],
  [ r'ParaSail', None, False ],
  [ r'PARI/GP', None, False ],
  [ r'Pascal', None, False ],
  [ r'Pawn', None, False ],
  [ r'PCASTL', None, False ],
  [ r'PCF', None, False ],
  [ r'PEARL', None, False ],
  [ r'PeopleCode', None, False ],
  [ r'Perl', None, False ],
  [ r'PDL', None, False ],
  [ r'PHP', None, False ],
  [ r'Phrogram', None, False ],
  [ r'Pico', None, False ],
  [ r'Pict', None, False ],
  [ r'Pike', None, False ],
  [ r'PIKT', None, False ],
  [ r'PILOT', None, False ],
  [ r'Pipelines', None, False ],
  [ r'Pizza', None, False ],
  [ r'PL-11', None, False ],
  [ r'PL/0', None, False ],
  [ r'PL/B', None, False ],
  [ r'PL/C', None, False ],
  [ r'PL/I', None, False ],
  [ r'PL/M', None, False ],
  [ r'PL/P', None, False ],
  [ r'PL/SQL', None, False ],
  [ r'PL360', None, False ],
  [ r'PLANC', None, False ],
  [ r'Plankalk', None, False ],
  [ r'Planner', None, False ],
  [ r'PLEX', None, False ],
  [ r'PLEXIL', None, False ],
  [ r'Plus', None, False ],
  [ r'POP-11', None, False ],
  [ r'PostScript', None, False ],
  [ r'PortablE', None, False ],
  [ r'Powerhouse', None, False ],
  [ r'PowerBuilder', None, False ],
  [ r'PowerShell', None, False ],
  [ r'PPL', None, False ],
  [ r'Processing', None, False ],
  [ r'Processing.js', None, False ],
  [ r'Prograph', None, False ],
  [ r'PROIV', None, False ],
  [ r'Prolog', None, False ],
  [ r'PROMAL', None, False ],
  [ r'Promela', None, False ],
  [ r'PROSE', None, False ],
  [ r'PROTEL', None, False ],
  [ r'ProvideX', None, False ],
  [ r'Pro*C', None, False ],
  [ r'Pure', None, False ],
  [ r'Python', None, False ],
  [ r'Q', None, False ],
  [ r'Qalb', None, False ],
  [ r'Qi', None, False ],
  [ r'QtScript', None, False ],
  [ r'QuakeC', None, False ],
  [ r'QPL', None, False ],
  [ r'R', None, False ],
  [ r'R\+\+', None, False ],
  [ r'Racket', None, False ],
  [ r'RAPID', None, False ],
  [ r'Rapira', None, False ],
  [ r'Ratfiv', None, False ],
  [ r'Ratfor', None, False ],
  [ r'rc', None, False ],
  [ r'REBOL', None, False ],
  [ r'Red', None, False ],
  [ r'Redcode', None, False ],
  [ r'REFAL', None, False ],
  [ r'Reia', None, False ],
  [ r'Revolution', None, False ],
  [ r'rex', None, False ],
  [ r'REXX', None, False ],
  [ r'Rlab', None, False ],
  [ r'RobotC', None, False ],
  [ r'ROOP', None, False ],
  [ r'RPG', None, False ],
  [ r'RPL', None, False ],
  [ r'RSL', None, False ],
  [ r'RTL/?2', None, False ],
  [ r'Ruby', None, False ],
  [ r'Rust', None, False ],
  [ r'S', None, False ],
  [ r'S', None, False ],
  [ r'S2', None, False ],
  [ r'S3', None, False ],
  [ r'S-Lang', None, False ],
  [ r'S-PLUS', None, False ],
  [ r'SA-C', None, False ],
  [ r'SabreTalk', None, False ],
  [ r'SAIL', None, False ],
  [ r'SALSA', None, False ],
  [ r'SAM76', None, False ],
  [ r'SAS', None, False ],
  [ r'SASL', None, False ],
  [ r'Sather', None, False ],
  [ r'Sawzall', None, False ],
  [ r'SBL', None, False ],
  [ r'Scala', None, False ],
  [ r'Scheme', None, False ],
  [ r'Scilab', None, False ],
  [ r'Scratch', None, False ],
  [ r'Script ?.NET', None, False ],
  [ r'Sed', None, False ],
  [ r'Seed7', None, False ],
  [ r'Self', None, False ],
  [ r'SenseTalk', None, False ],
  [ r'SequenceL', None, False ],
  [ r'SETL', None, False ],
  [ r'Shift Script', None, False ],
  [ r'SIMPOL', None, False ],
  [ r'SIMSCRIPT', None, False ],
  [ r'Simula', None, False ],
  [ r'Simulink', None, False ],
  [ r'SISAL', None, False ],
  [ r'SLIP', None, False ],
  [ r'SMALL', None, False ],
  [ r'Smalltalk', None, False ],
  [ r'Small Basic', None, False ],
  [ r'SML', None, False ],
  [ r'SNOBOL', None, False ],
  [ r'SPITBOL', None, False ],
  [ r'Snowball', None, False ],
  [ r'SOL', None, False ],
  [ r'Span', None, False ],
  [ r'SPARK', None, False ],
  [ r'SPIN', None, False ],
  [ r'SP/?k', None, False ],
  [ r'SPS', None, False ],
  [ r'Squeak', None, False ],
  [ r'Squirrel', None, False ],
  [ r'SR', None, False ],
  [ r'S/?SL', None, False ],
  [ r'Stackless Python', None, False ],
  [ r'Starlogo', None, False ],
  [ r'Strand', None, False ],
  [ r'Stata', None, False ],
  [ r'Stateflow', None, False ],
  [ r'Subtext', None, False ],
  [ r'SuperCollider', None, False ],
  [ r'SuperTalk', None, False ],
  [ r'Swift', None, False ],
  [ r'SYMPL', None, False ],
  [ r'SyncCharts', None, False ],
  [ r'SystemVerilog', None, False ],
  [ r'T', None, False ],
  [ r'T', None, False ],
  [ r'TACL', None, False ],
  [ r'TACPOL', None, False ],
  [ r'TADS', None, False ],
  [ r'TAL', None, False ],
  [ r'Tcl', None, False ],
  [ r'Tea', None, False ],
  [ r'TECO', None, False ],
  [ r'TELCOMP', None, False ],
  [ r'TeX', None, False ],
  [ r'TEX', None, False ],
  [ r'TIE', None, False ],
  [ r'Timber', None, False ],
  [ r'TMG', None, False ],
  [ r'Tom', None, False ],
  [ r'TOM', None, False ],
  [ r'Topspeed', None, False ],
  [ r'TPU', None, False ],
  [ r'Trac', None, False ],
  [ r'TTM', None, False ],
  [ r'T-SQL', None, False ],
  [ r'TTCN', None, False ],
  [ r'Turing', None, False ],
  [ r'TUTOR', None, False ],
  [ r'TXL', None, False ],
  [ r'TypeScript', None, False ],
  [ r'T(urbo )?C\+\+', None, False ],
  [ r'U', None, False ],
  [ r'Ubercode', None, False ],
  [ r'UCSD Pascal', None, False ],
  [ r'Umple', None, False ],
  [ r'Unicon', None, False ],
  [ r'Uniface', None, False ],
  [ r'UNITY', None, False ],
  [ r'Unix shell', None, False ],
  [ r'UnrealScript', None, False ],
  [ r'V', None, False ],
  [ r'Vala', None, False ],
  [ r'VBA', None, False ],
  [ r'VBScript', None, False ],
  [ r'Verilog', None, False ],
  [ r'VHDL', None, False ],
  [ r'Visual Basic', None, False ],
  [ r'Visual Basic .NET', None, False ],
  [ r'Visual DataFlex', None, False ],
  [ r'Visual DialogScript', None, False ],
  [ r'Visual Fortran', None, False ],
  [ r'Visual FoxPro', None, False ],
  [ r'Visual J\+\+', None, False ],
  [ r'Visual J#', None, False ],
  [ r'Visual Objects', None, False ],
  [ r'Visual Prolog', None, False ],
  [ r'VSXu', None, False ],
  [ r'Vvvv', None, False ],
  [ r'W', None, False ],
  [ r'WATFIV, WATFOR', None, False ],
  [ r'WebDNA', None, False ],
  [ r'WebQL', None, False ],
  [ r'(Windows )?PowerShell', None, False ],
  [ r'PowerShell', None, False ],
  [ r'Winbatch', None, False ],
  [ r'Wolfram', None, False ],
  [ r'Wyvern', None, False ],
  [ r'X(\+\+)?', None, False ],
  [ r'X#', None, False ],
  [ r'X10', None, False ],
  [ r'XBL', None, False ],
  [ r'XC', None, False ],
  [ r'XMOS', None, False ],
  [ r'xHarbour', None, False ],
  [ r'XL', None, False ],
  [ r'Xojo', None, False ],
  [ r'XOTcl', None, False ],
  [ r'XPL', None, False ],
  [ r'XPL0', None, False ],
  [ r'XQuery', None, False ],
  [ r'XSB', None, False ],
  [ r'XSLT', None, False ],
  [ r'Y', None, False ],
  [ r'Yorick', None, False ],
  [ r'YQL', None, False ],
  [ r'Z', None, False ],
  [ r'Z notation', None, False ],
  [ r'Zeno', None, False ],

  # Some non-programming-languages just in case.
  [ r'XML', None, False ],
  [ r'', None, False ],
  [ r'', None, False ],
  [ r'', None, False ],
  [ r'', None, False ],
  [ r'', None, False ],


]

TEMPLATE_KEY_DATA = {}

HTML_ESCAPE_TABLE = {
  "&": "&amp;",
  '"': "&quot;",
  "'": "&apos;",
  ">": "&gt;",
  "<": "&lt;",
}

# Simple URL regex. Deprecated now in favor of just accepting anything.
URL_REGEX = re.compile(
  r"""\b((?#protocol)https?|ftp)://((?#domain)[-A-Z0-9.]+)((?#file)/[-A-Z0-9+&@#/%=~_|!:,.;]*)?((?#parameters)\?[A-Z0-9+&@#/%=~_|!:,.;]*)?""",
  re.IGNORECASE | re.VERBOSE)

EMAIL_REGEX = re.compile(
  r"""(?:[a-z0-9!#$%&'*+/=?^_`{|}~-]+(?:\.[a-z0-9!#$%&'*+/=?^_`{|}~-]+)*|"(?:[\x01-\x08\x0b\x0c\x0e-\x1f\x21\x23-\x5b\x5d-\x7f]|\\[\x01-\x09\x0b\x0c\x0e-\x7f])*")@(?:(?:[a-z0-9](?:[a-z0-9-]*[a-z0-9])?\.)+[a-z0-9](?:[a-z0-9-]*[a-z0-9])?|\[(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?|[a-z0-9-]*[a-z0-9]:(?:[\x01-\x08\x0b\x0c\x0e-\x1f\x21-\x5a\x53-\x7f]|\\[\x01-\x09\x0b\x0c\x0e-\x7f])+)\])""",
  re.IGNORECASE | re.VERBOSE)

DEBUG = False

ERROR_LIST = []

# Exception class of our own!
class ContactError(Exception):
  def __init__(self, value):
    self.value = value
  def __str__(self):
    return repr(self.value)

def send_email(form_name, form_email, form_company, form_website, form_message,
               form_antispam):
  """Does the work of sending an e-mail message, the point of this whole thing!
  """
  global ERROR_LIST

  try:
    msg = MIMEMultipart()
    from_full = form_name + ' <' + form_email + '>'
    msg['From'] = from_full
    msg['To'] = COMMASPACE.join(NOTIFICATION_EMAILS)
    msg['Date'] = formatdate(localtime=True)
    msg['Subject'] = 'Contact from ' + form_name + ' on HireBrett.com'

    message = """Name: %s
E-mail Address: %s
Company Name: %s
Website: %s
Anti-Spam: %s
Message:

%s""" % (form_name, form_email, form_company, form_website, form_antispam,
           form_message)

    msg.attach(MIMEText(message))

    smtp = smtplib.SMTP(MAIL_SERVER)
    smtp.sendmail(from_full, NOTIFICATION_EMAILS[0], msg.as_string())
    smtp.close()

  except Exception, e:
    # Oops! There was a problem sending the mail. We can't do anything now, but
    # since the user is having trouble, we'll let them know how they can reach
    # us via e-mail without this form.
    ERROR_LIST, TEMPLATE_KEY_DATA = add_error( \
                           ERROR_LIST, TEMPLATE_KEY_DATA, '',
                          'There seems to be an internal problem with our mail server. Sorry about that! Please try again later. Alternatively, you can send your e-mail directly to <a href=\"mailto:bgmoser@codexterous.com\">brett@hirebrett.com</a>.',
                          'value')

def html_escape(text):
  """Produce entities within text."""
  return "".join(HTML_ESCAPE_TABLE.get(c,c) for c in text)

def find_replacement(match):
  """Callback for fill_template."""
  for k in TEMPLATE_KEY_DATA.keys():
    if k.lower() == match.group(1).lower():
      return TEMPLATE_KEY_DATA[match.group(1).lower()]
  return ''

def fill_template(file_data):
  """Replace template "variables" with entries in TEMPLATE_KEY_DATA."""
  reobj = re.compile("{{(.+?)}}", re.IGNORECASE)
  res = reobj.sub(find_replacement, file_data)
  return res

def add_error_class(data, name):
  data['class_' + name.lower()] = ' class="error"'
  return data

def add_value(data, name, value, no_escape=False):
  if no_escape:
    data['value_' + name.lower()] = ' value="' + value + '"'
  else:
    data['value_' + name.lower()] = ' value="' + html_escape(value) + '"'
  return data

def add_textarea(data, name, value, no_escape=False):
  if no_escape:
    data['textarea_' + name.lower()] = value
  else:
    data['textarea_' + name.lower()] = html_escape(value)
  return data

def add_error(error_list, vars_list, field, message, type):
  add_error_class(vars_list, field)
  error_list.append(message)
  return error_list, vars_list

def main():
  global DEBUG, TEMPLATE_KEY_DATA, ERROR_LIST

  #print 'Content-Type: text/html'
  #print

  # Set up logging
  logging.basicConfig(filename="hirebrett_contact.py.log", level=logging.DEBUG)

  post_data = {}
  content_length = 0
  content = ''
  query_string = ''
  request_method = ''
  # For whatever stupid reason, cgi.FieldStorage is not at all reading POST data
  # for us. At all. There's probably a header it's looking for somewhere that
  # isn't being passed upstream by the nginx load balancers, or something like
  # that.
  for name, value in os.environ.items():
    if name.lower() == 'http_content_length':
      content_length = int(value)
      content = sys.stdin.read(content_length)
    elif name.lower() == 'http_request_method':
      request_method = value

    # Dump environment into template variables as well.
    TEMPLATE_KEY_DATA[name.lower()] = value

  # We're going to fake out cgi.FieldStorage by appending the post data to the
  # query string.
  if len(content):
    if len(query_string):
      content = '&' + content
    query_string += content

  # Read the GET and POST data right away - we need to see if debug mode is
  # enabled.
  post_data = cgi.FieldStorage(environ={'QUERY_STRING':query_string})
  #post_data = cgi.FieldStorage()

  # Check if debug mode is enabled.
  if post_data.has_key('dbg') and post_data.getvalue('dbg').lower() == 'y':
    import cgitb
    cgitb.enable()
    DEBUG = True

  # This is for our templating use. First, intelligently see if we can work out
  # our script filename and figure out where '..' is, otherwise just use PWD.
  # We do this because when we're debugging this we don't have that, but when
  # it's running on a web server, we don't have a correct working directory.
  script_filename = os.getenv('SCRIPT_FILENAME')
  contact_filename = None
  success_filename = None
  final_output = None
  final_status = '200 OK'
  errors = []
  if script_filename:
    # If we're running on www.codexterous.com cgi-bin, use the absolute path for
    # it. This is just temporary for debugging purposes.
    if 'www.codexterous.com' in script_filename:
      contact_filename = '/usr/local/www/sites/www.hirebrett.com/htdocs/contact.html'
      success_filename = '/usr/local/www/sites/www.hirebrett.com/htdocs/contact_success.html'
    else:
      base = os.path.normpath( \
                           os.path.join( \
                             os.path.split( \
                               script_filename)[0], '..'))
      contact_filename = base + '/contact.html'
      success_filename = base + '/contact_success.html'
  else:
    contact_filename = '../contact.html'
    success_filename = '../contact_success.html'

  if not os.path.exists(contact_filename) or \
     not os.path.exists(success_filename):
    raise ContactError, "Unable to open the HTML for the contact form! (file=" + contact_filename + ")"

  # Before doing anything, we need to decide which template we'll be using.
  # To that end, we need to process post data first to see if this will be a
  # successful use of the form.
  sent_email = False
  if request_method.lower() == 'post':
    form_name = post_data.getfirst('name') or ''
    form_email = post_data.getfirst('email') or ''
    form_company = post_data.getfirst('company') or ''
    form_website = post_data.getfirst('website') or ''
    form_message = post_data.getfirst('message') or ''
    form_antispam = post_data.getfirst('antispam') or ''

    # Check all of this for errors.
    if not re.match(EMAIL_REGEX, form_email):
      ERROR_LIST, TEMPLATE_KEY_DATA = add_error( \
                             ERROR_LIST, TEMPLATE_KEY_DATA, 'email',
                            'Please enter a valid e-mail address.', 'value')

    if len(form_website) and not re.match('.{5,}', form_website):
      ERROR_LIST, TEMPLATE_KEY_DATA = add_error( \
                             ERROR_LIST, TEMPLATE_KEY_DATA, 'website',
                            'Please enter a valid URL for your website address.', 'value')

    if not re.match('.{3,}', form_name):
      ERROR_LIST, TEMPLATE_KEY_DATA = add_error( \
                             ERROR_LIST, TEMPLATE_KEY_DATA, 'name',
                            'Please enter a valid name.', 'value')

    if len(form_company) and not re.match('.{3,}', form_company):
      ERROR_LIST, TEMPLATE_KEY_DATA = add_error( \
                             ERROR_LIST, TEMPLATE_KEY_DATA, 'company',
                            'Please enter a valid company name.', 'value')

    if not re.match('.{5,}', form_message):
      ERROR_LIST, TEMPLATE_KEY_DATA = add_error( \
                             ERROR_LIST, TEMPLATE_KEY_DATA, 'message',
                            'Please enter a valid message, or perhaps a slightly longer one.', 'textarea')

    # Let's check the anti-spam.
    valid_language = False
    for r in LANGUAGES:
      if re.match('^' + r[0] + '$', form_antispam, re.IGNORECASE):
        # We have a valid language!
        valid_language = True

        # Sometimes, we have cheeky messages for the user about their language
        # of choice!
        if r[1]:
          TEMPLATE_KEY_DATA['antispam_comment'] = '<p id=\"antispam\">P.S.: ' + r[1] + '</p>'

    if (not valid_language) or (not len(form_antispam)): # the length check is
                                                     # because while developing
                                                     # this I had empty regexes
      ERROR_LIST, TEMPLATE_KEY_DATA = add_error( \
                             ERROR_LIST, TEMPLATE_KEY_DATA, 'antispam',
                            'Please enter a valid programming language for the anti-spam question! If you\'re stuck, pick one from <a href="/">the homepage</a> or from Wikipedia\'s <a href="http://en.wikipedia.org/wiki/List_of_programming_languages">list of programming languages</a>.', 'textarea')

    # Okay, can we send the e-mail? If there are no errors, we can indeed!
    if not len(ERROR_LIST):
      send_email(form_name, form_email, form_company, form_website,
                 form_message, form_antispam)

    # If send_email has a problem, it'll add to the error list. So check
    # that again.
    if not len(ERROR_LIST):
      # We can show the success page!
      sent_email = True

  # Next, we'll actually show the completed page. First determine which template
  # we'll be using.
  template_filename = contact_filename
  if sent_email:
    template_filename = success_filename

  with open(template_filename, 'r') as f:
    # Do what we can in an inner try block - we can be a little more pretty that
    # way with how we deal with internal errors.
    try:

      # First, suck in the template.
      data = f.read()

      # Insert all user input.
      for k in post_data.keys():
        if k.lower() == 'message':
          TEMPLATE_KEY_DATA = add_textarea(TEMPLATE_KEY_DATA, k.lower(), post_data.getfirst(k))
        else:
          TEMPLATE_KEY_DATA = add_value(TEMPLATE_KEY_DATA, k.lower(), post_data.getfirst(k))

      # Final check to add to error list.
      if len(ERROR_LIST):
        # There were errors!
        error_value = "<p class=\"error\"><strong>Oops, there was a problem with your submission! Here's what went wrong:</strong></p><ul class=\"error\">"

        # Normally I'd html escape these, but the errors come from safe within
        # this application, and nowhere do they reproduce user input. Also, we
        # sometimes use HTML in our messages.
        for e in ERROR_LIST:
          error_value += '<li>' + e + '</li>'
        error_value += '</ul>'
        TEMPLATE_KEY_DATA['errors'] = error_value

      final_output = fill_template(data)
      #print data
    except Exception, e:
      final_status = '500 Internal Server Error'
      final_output = 'Oops, there was a problem! ' + str(e)

    # Print our headers
    print 'Status: ' + final_status
    print "Content-Type: text/html"
    print
    print final_output


  pass

if __name__ == '__main__':
  try:
    main()
  except Exception, e:
    logging.error("Exception was thrown: " + str(e))
    print "\r\n".join(["Status: 500 Internal Server Error",
                       "Content-Type: text/plain\r\n",
                       "".join(["Exception was thrown: ", str(e)])])

